'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jcampconverter = require('jcampconverter');
var nmrProcessing = require('nmr-processing');

/**
 * Returns a list of likely nuclei based on an experiment string
 * This is really an hypothesis and should not be used
 * @param {string} experiment
 * @return {string[]}
 */

function getNucleusFrom2DExperiment(experiment) {
  if (typeof experiment !== 'string') {
    return [];
  }
  experiment = experiment.toLowerCase();
  if (experiment.includes('jres')) {
    return ['1H'];
  }
  if (experiment.includes('hmbc') || experiment.includes('hsqc')) {
    return ['1H', '13C'];
  }
  return ['1H', '1H'];
}

/**
 * Returns an experiment string based on a pulse sequence
 * @param {string} pulse
 * @return {string}
 */
function getSpectrumType(meta = {}, info = {}) {
  if (meta === null) meta = {};
  if (typeof meta === 'string') {
    meta = { pulse: meta };
  }

  let spectyp;
  if (Array.isArray(info.$SPECTYP)) {
    spectyp = (info.$SPECTYP[0] || '').replace(/^<(.*)>$/, '$1').toLowerCase();
  } else {
    spectyp = (info.$SPECTYP || '').replace(/^<(.*)>$/, '$1').toLowerCase();
  }

  if (spectyp) return spectyp;

  let pulse = meta.pulse;
  if (typeof pulse !== 'string') {
    return '';
  }

  pulse = pulse.toLowerCase();
  if (pulse.includes('zg')) {
    return '1d';
  }

  if (
    pulse.includes('hsqct') ||
    (pulse.includes('invi') && (pulse.includes('ml') || pulse.includes('di')))
  ) {
    return 'hsqctocsy';
  }

  if (pulse.includes('hsqc') || pulse.includes('invi')) {
    return 'hsqc';
  }

  if (
    pulse.includes('hmbc') ||
    (pulse.includes('inv4') && pulse.includes('lp'))
  ) {
    return 'hmbc';
  }

  if (pulse.includes('cosy')) {
    return 'cosy';
  }

  if (pulse.includes('jres')) {
    return 'jres';
  }

  if (
    pulse.includes('tocsy') ||
    pulse.includes('mlev') ||
    pulse.includes('dipsi')
  ) {
    return 'tocsy';
  }

  if (pulse.includes('noesy')) {
    return 'noesy';
  }

  if (pulse.includes('roesy')) {
    return 'roesy';
  }

  if (pulse.includes('dept')) {
    return 'dept';
  }

  if (pulse.includes('jmod') || pulse.includes('apt')) {
    return 'aptjmod';
  }

  return '';
}

function getMetaData(info, meta) {
  const metadata = {
    dimension: 1,
    nucleus: [],
    isFid: false,
    isFt: false,
    isComplex: false,
  };

  maybeAdd(metadata, 'title', info.TITLE);
  maybeAdd(metadata, 'solvent', info['.SOLVENTNAME']);
  maybeAdd(
    metadata,
    'pulse',
    info['.PULSESEQUENCE'] || info['.PULPROG'] || meta.PULPROG,
  );
  maybeAdd(metadata, 'experiment', getSpectrumType(metadata, info));
  maybeAdd(metadata, 'temperature', parseFloat(meta.TE || info['.TE']));
  maybeAdd(metadata, 'frequency', parseFloat(info['.OBSERVEFREQUENCY']));
  maybeAdd(metadata, 'type', info.DATATYPE);
  maybeAdd(metadata, 'probe', meta.PROBHD);
  if (meta.FNTYPE !== undefined) {
    maybeAdd(metadata, 'acquisitionMode', parseInt(meta.FNTYPE));
  }
  maybeAdd(metadata, 'expno', parseInt(meta.EXPNO));
  if (metadata.type) {
    if (metadata.type.toUpperCase().indexOf('FID') >= 0) {
      metadata.isFid = true;
    } else if (metadata.type.toUpperCase().indexOf('SPECTRUM') >= 0) {
      metadata.isFt = true;
    }
  }

  if (info['.NUCLEUS']) {
    metadata.nucleus = info['.NUCLEUS'].split(',').map((nuc) => nuc.trim());
  } else if (info['.OBSERVENUCLEUS']) {
    metadata.nucleus = [info['.OBSERVENUCLEUS'].replace(/[^A-Za-z0-9]/g, '')];
  } else {
    metadata.nucleus = getNucleusFrom2DExperiment(metadata.experiment);
  }

  metadata.dimension = metadata.nucleus.length;

  if (info.SYMBOL) {
    let symbols = info.SYMBOL.split(/[, ]+/);
    if (symbols.includes('R') && symbols.includes('I')) {
      metadata.isComplex = true;
    }
  }

  if (meta.DATE) {
    metadata.date = new Date(meta.DATE * 1000).toISOString();
  }
  return metadata;
}

function maybeAdd(obj, name, value) {
  if (value !== undefined) {
    if (typeof value === 'string') {
      if (value.startsWith('<') && value.endsWith('>')) {
        value = value.substring(1, value.length - 2);
      }
      obj[name] = value.trim();
    } else {
      obj[name] = value;
    }
  }
}

const fromInfo = getMetaData;

/**
 * Object containing parsed metadata
 * @name NMRMetadata
 * @typedef {object} NMRMetadata
 * @property {number} dimension
 * @property {number[]} nucleus
 * @property {string} title
 * @property {string} solvent
 * @property {string} pulse
 * @property {string} experiment
 * @property {number} temperature - Temperature in Kelvin
 * @property {number} frequency
 * @property {string} probe
 * @property {string} acquisitionMode
 * @property {number} expno - Experience number
 * @property {string} date - Date in ISO string format
 * @property {object} ranges
 */

const defaultOptions = {
  computeRanges: false,
};

const defaultRangesOptions = {
  integrationSum: 100,
  clean: 0.4,
  compile: true,
  integralType: 'sum',
};

const defaultPeaksOptions = {
  thresholdFactor: 0.85,
};

/**
 * Returns a metadata object from JCAMP
 * @param {string} jcampData
 * @param {object} [options={}]
 * @param {boolean} [options.computeRanges=false]
 * @param {number} [options.ranges] - options for ranges computation
 * @return {NMRMetadata} metadata
 */
function fromJcamp(jcampData, options = {}) {
  options = {
    ...defaultOptions,
    ...options,
  };

  const jcampString = jcampData.toString();
  const parsedJcamp = jcampconverter.convert(jcampString, {
    keepRecordsRegExp: /.*/,
    canonicMetadataLabels: true,
    withoutXY: false,
  }).flatten[0];

  let metadata = getMetaData(parsedJcamp.info, parsedJcamp.meta);

  if (
    options.computeRanges &&
    metadata.isFt &&
    metadata.dimension === 1 &&
    metadata.nucleus[0] === '1H'
  ) {
    let { ranges = {}, impurities = {}, peakPicking = {} } = options;

    const rangesOptions = { ...defaultRangesOptions, ...ranges };
    const peaksOptions = { ...defaultPeaksOptions, ...peakPicking };

    if (metadata.solvent) {
      impurities.solvent = metadata.solvent;
    }

    metadata.range = nmrProcessing.xyAutoRangesPicking(parsedJcamp.spectra[0].data, {
      impurities,
      ranges: rangesOptions,
      peakPicking: peaksOptions,
    });
  }

  return metadata;
}

exports.fromInfo = fromInfo;
exports.fromJcamp = fromJcamp;
exports.getSpectrumType = getSpectrumType;
